% -------------------------------------------------------------------------
% This code is used to create coordiantes for describing the surface of 
% assymetric (four digit) NACA airfoils
% -------------------------------------------------------------------------
% The four digits in the name NACAmprr describe:
% m = maximum camber, as a fraction of chord length (*100)
% p = location of maximum camber, as a fraction of chord (*10)
% rr = percentage number for the ratio thickness/chord (t/c), where 
%      t = maximum thickness and c = chord (length of airfoil) [m] (*100) 
% 
% Developed by: Daniel Lindblad, MPENM-2, Chalmers University of Technology


% -------------------------------------------------------------------------
% Specification of parameters
% -------------------------------------------------------------------------

m = 0.04;   % Maximum camber
p = 0.4;    % Location of maximum camber
t = 0.12;   % Maximum thickness 
c = 1;      % Chord length

N = 30;     % Amount of intervals along chord to divide geometry in if
            % simple grading is applied
            
dx_0 = 0.05*c;  % If finer grading close to leading edge is applied, dx_0  
                % defines the grading used further back on the airfoil.

% Calculation of NACA number for the airfol

NACA = 1000*(100*m) + 100*(10*p) + 100*t;

% -------------------------------------------------------------------------
% Calculaiting the profile
% -------------------------------------------------------------------------

% Create the mesh along the chord of the airfoil

% Simple grading:
% x = 0:c/N:c;        

% Varying grading:
% Create function defining step size in x directon, for grading of airfoil
f_delta = @(x) (1 - 0.9*exp(-(2*x./(p*c)).^2)).*dx_0; 

x = 0;          % Initialize the x coordinates
k = 1;          % Initialize the loop index
while x(k) < c
    x = [x, x(k) + f_delta(x(k))];
    k = k+1;
end
x(k) = c;



% Calculate the thickness of a symmetrical airfoil
y_t = y_t_calc(x, t, c);

% Calculate the displacement of the camber
y_c = y_c_calc(x, m, p, c);

% Calculate the final wing profile as a file of points
[C_l, C_u] = profile_calc(y_t, y_c, x, m, p, c);

% Fix trailing edge, so that both the top and bottom side of the airfoil
% coincide at the trailing edge.
C_l(:,end) = [c;0];
C_u(:,end) = [c;0];

% Plot the resulting wing profile
figure(1)

plot(C_l(1, :), C_l(2, :), 'r', 'linewidth', 2);
hold on
plot(C_u(1, :), C_u(2, :), 'r', 'linewidth', 2);
plot(x, y_c, 'g', 'linewidth', 2);
axis equal
grid on
name = num2str(NACA);
title(name)

% -------------------------------------------------------------------------
% Creating a Airfoil.data file to be read by fortran routine that creates 
% the blockMesh disc that will be used in OpenFOAM
% -------------------------------------------------------------------------

n_points = length(x);   % Find the amount of coordinates used to describe 
                        % one side of the airfoil
             
fileID = fopen('Airfoil.data','w');
fprintf(fileID, '%3.0f  %3.0f\r\n\r\n', [n_points; n_points]);
fprintf(fileID, '   %8.6f  %8.6f\r\n', C_u);
fprintf(fileID, '\r\n')
fprintf(fileID, '   %8.6f  %8.6f\r\n', C_l);
fclose(fileID);

% -------------------------------------------------------------------------
% Rotation of airfoil, to inspect it at various angles of attack
% -------------------------------------------------------------------------


% This part takes the airfoil and rotates it around the point of maximum
% camber to make ot possible to change the angle of attack for the airfoil

% Define the angle of attack (degrees)
alpha = -5;
alpha = alpha/360*2*pi;

% Calculate the rotation matrix
R = [cos(alpha) -sin(alpha); sin(alpha) cos(alpha)];

% Define the camber line
Camber = [x; y_c];

% Transform coordinates into coordinate system with origin at point of
% maximum camber 
C_l(1,:) = C_l(1,:) - c*p*ones(size(x));
C_u(1,:) = C_u(1,:) - c*p*ones(size(x));
Camber(1,:) = Camber(1,:) - c*p*ones(size(x));

% Rotate the system
for i = 1:length(x)
    C_l(:,i) = R*C_l(:,i);
    C_u(:,i) = R*C_u(:,i);
    Camber(:,i) = R*Camber(:,i);
end

% Transform back into original coordinate system
C_l(1,:) = C_l(1,:) + c*p*ones(size(x));
C_u(1,:) = C_u(1,:) + c*p*ones(size(x));
Camber(1,:) = Camber(1,:) + c*p*ones(size(x));

% Plot the resulting wing profile
figure(2)

plot(C_l(1, :), C_l(2, :), 'r', 'linewidth', 2);
hold on
plot(C_u(1, :), C_u(2, :), 'r', 'linewidth', 2);
plot(Camber(1,:), Camber(2,:), 'g', 'linewidth', 2);
axis equal
grid on
name = num2str(NACA);
title(name)












