/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011-2012 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::incompressible::LESModels::kOmegaSSTDES

Group
    grpIcoLESTurbulence

Description
    k-Omega-SST-DES LES turbulence model for incompressible flows
    based on:

    "Ten Years of Industrial Experience with the SST Turbulence Model".
    Turbulence Heat and Mass Transfer 4.
    F. R. Menter, M. Kuntz, R. Langtry.

    A note on the implementation with respect to the paper:

    - The transport equations implemented are divided by rho, as opposed to the
      presentation of them in the paper, since the flow is incompressible

    - There is a choice to use the model in DES (Detached Eddy Simulation) mode
      or DDES (Delayed Detached Eddy Simulations) mode. The latter protects the 
      boundary layer from being resolved with LES and is used by default. The 
      other part is commented away.

    - Model constants are not allways named the same as in the paper, see comments
      in declaration of constants below.

-------------------------------------------------------------------------------
DISCLAIMER:
    This is a student project work in the course CFD with OpenSource software 
    taught by Hakan Nilsson, Chalmers University of Technology, Gothenburg, Sweden. 
------------------------------------------------------------------------------- 

SourceFiles
    kOmegaSSTDES.C

\*---------------------------------------------------------------------------*/

#ifndef kOmegaSSTDES_H
#define kOmegaSSTDES_H

#include "LESModel.H"
#include "volFields.H"
#include "wallDist.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace incompressible
{
namespace LESModels
{

/*---------------------------------------------------------------------------*\
                        Class kOmegaSSTDES Declaration
\*---------------------------------------------------------------------------*/

class kOmegaSSTDES
:
    public LESModel
{
    // Private Member Functions

        //- Update sub-grid scale fields
        void updateSubGridScaleFields(const volScalarField& D);

        // Disallow default bitwise copy construct and assignment
        kOmegaSSTDES(const kOmegaSSTDES&);
        kOmegaSSTDES& operator=(const kOmegaSSTDES&);


protected:

    // Protected data

        // Model constants

            dimensionedScalar alphaK1_;		// sigma_k_1
            dimensionedScalar alphaK2_;		// sigma_k_2

            dimensionedScalar alphaOmega1_;	// sigma_omega_1
            dimensionedScalar alphaOmega2_;	// sigma_omega_2

            dimensionedScalar gamma1_;		// alpha_1
            dimensionedScalar gamma2_;		// alpha_2

            dimensionedScalar beta1_;		// beta_1
            dimensionedScalar beta2_;		// beta_2

            dimensionedScalar betaStar_;	// beta^star

            dimensionedScalar a1_;		// a_1
            dimensionedScalar c1_;		// c_1 
            // dimensionedScalar Cs_;

            // dimensionedScalar alphaPhi_;
            // dimensionedScalar zetaTilda2_;
            // dimensionedScalar FSAS_;

            dimensionedScalar omegaMin_;

            wallDist y_;
            // dimensionedScalar Cmu_;
            // dimensionedScalar kappa_;

	    dimensionedScalar CDES_;		// C_DES


        // Fields

            volScalarField k_;
            volScalarField omega_;
            volScalarField nuSgs_;


    // Protected Member Functions

	/*
        tmp<volScalarField> Lvk2
        (
            const volScalarField& S2
        ) const; 
	*/

	tmp<volScalarField> F1(const volScalarField& CDkOmega) const;
        tmp<volScalarField> F2() const;

        tmp<volScalarField> blend
        (
            const volScalarField& F1,
            const dimensionedScalar& psi1,
            const dimensionedScalar& psi2
        ) const
        {
            return F1*(psi1 - psi2) + psi2;
        }

	// Choose DES or DDES by commenting the non desired option

	// tmp<volScalarField> FDES() const;	

	tmp<volScalarField> FDDES(const volScalarField& FS) const;	


        tmp<volScalarField> alphaK		// sigma_k
        (
            const volScalarField& F1
        ) const
        {
            return blend(F1, alphaK1_, alphaK2_);
        }

        tmp<volScalarField> alphaOmega		// sigma_omega
        (
             const volScalarField& F1
        ) const
        {
            return blend(F1, alphaOmega1_, alphaOmega2_);
        }

        tmp<volScalarField> beta		// beta
        (
            const volScalarField& F1
        ) const
        {
            return blend(F1, beta1_, beta2_);
        }

        tmp<volScalarField> gamma		// alpha
        (
            const volScalarField& F1
        ) const
        {
            return blend(F1, gamma1_, gamma2_);
        }


public:

    //- Runtime type information
    TypeName("kOmegaSSTDES");


    // Constructors

        //- Construct from components
        kOmegaSSTDES
        (
            const volVectorField& U,
            const surfaceScalarField& phi,
            transportModel& transport,
            const word& turbulenceModelName = turbulenceModel::typeName,
            const word& modelName = typeName
        );


    //- Destructor
    virtual ~kOmegaSSTDES()
    {}


    // Member Functions

        //- Return SGS kinetic energy
        virtual tmp<volScalarField> k() const
        {
            return k_;
        }

        //- Return omega
        virtual tmp<volScalarField> omega() const
        {
            return omega_;
        }

        //- Return the effective diffusivity for k
        tmp<volScalarField> DkEff(const volScalarField& F1) const
        {
            return tmp<volScalarField>
            (
                 new volScalarField("DkEff", alphaK(F1)*nuSgs_ + nu())
            );
        }

        //- Return the effective diffusivity for omega
        tmp<volScalarField> DomegaEff(const volScalarField& F1) const
        {
            return tmp<volScalarField>
            (
                new volScalarField("DomegaEff", alphaOmega(F1)*nuSgs_ + nu())
            );
        }

        //- Return sub-grid disipation rate
        virtual tmp<volScalarField> epsilon() const;

        //- Return SGS viscosity
        virtual tmp<volScalarField> nuSgs() const
        {
            return nuSgs_;
        }

        //- Return the sub-grid stress tensor.
        virtual tmp<volSymmTensorField> B() const;

        //- Return the effective sub-grid turbulence stress tensor
        //  including the laminar stress
        virtual tmp<volSymmTensorField> devReff() const;

        //- Return the deviatoric part of the effective sub-grid
        //  turbulence stress tensor including the laminar stress
        virtual tmp<fvVectorMatrix> divDevReff(volVectorField& U) const;

        //- Return the deviatoric part of the effective sub-grid
        //  turbulence stress tensor including the laminar stress
        virtual tmp<fvVectorMatrix> divDevRhoReff
        (
            const volScalarField& rho,
            volVectorField& U
        ) const;

        //- Solve the turbulence equations (k-w) and correct the turbulence
        //  viscosity
        virtual void correct(const tmp<volTensorField>& gradU);

        //- Read LESProperties dictionary
        virtual bool read();
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace LESModels
} // End namespace incompressible
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
